# ad40.tcl
#    setup for AD40 module



set frame AD40
set frametitle "AD40 Module Setup"
set icontitle  "AD40"

set RDOUTserver "cm62"
set RDOUThdwre  "CM62"


#
#   table(widget)          register
#

set table(INTMODE)      DSPIntMode
set table(TRIGSRC)      DSPTrigSrc
set table(TRIGCLK)      DSPTrigClk
set table(TRIGMODE)     DSPTrigMode
set table(TRIGCOUNT)    DSPTrigCount
set table(SAMPLERATE)   DSPSampleRate
set table(TRIGVOLTAGE)  DSPTrigVoltage

set widgets {TRIGSRC TRIGMODE TRIGCOUNT SAMPLERATE TRIGVOLTAGE}

#    for conversion from widget index to hardware value
set WtoIntMode(0) 1
set WtoIntMode(1) 2
set WtoIntMode(2) 4

set WtoTrigClk(0) 2
set WtoTrigClk(1) 3
set WtoTrigClk(2) 5

#   for conversion from hardware value to widget index
set IntModetoW(1) 0
set IntModetoW(2) 1
set IntModetoW(4) 2

set TrigClktoW(2) 0
set TrigClktoW(3) 1
set TrigClktoW(5) 2

set MaxTries 20


proc read-reg {register} {
  global RDOUTserver RDOUThdwre
  set reg /$RDOUTserver/$RDOUThdwre.$register
  set vl [EG "register :name '$reg' ?value"]
  return [expr $vl]
}

proc write-reg {register vl} {
  global RDOUTserver RDOUThdwre
  set reg /$RDOUTserver/$RDOUThdwre.$register
  EG "register :name '$reg' :value '$vl'"
}

proc update-widget {widget value} {
    :modify $widget :value $value
}

proc wait_for_completion {s} {
    global RDOUTserver MaxTries

    set tries 1

    while {$tries < $MaxTries} {

        set-footer "Waiting for action $s to complete ($tries)"

        set z [catch {EG "register :name '/$RDOUTserver/EXEC.Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "Unable to confirm state of action\n$m"
            clear-footer
            return
        }

        if {$m == 0} then {
            clear-footer
            return
        }
        incr tries
        after 500
    }

    midas-warning "Action $s has not completed\nGiving up waiting"
    clear-footer
}

proc click-update {} {
   global table widgets
   global RDOUTserver
   global IntModetoW TrigClktoW
  
#    request DA program to get parameters

   EG "register :name '/$RDOUTserver/EXEC.Action' :value '6'"
   EG "register :name '/$RDOUTserver/EXEC.Sig1' :value '0'"

   wait_for_completion "Getting Parameters"

   set-footer "Updating..."
   set-busy
   foreach w $widgets {
      set register $table($w)
      update-widget $w [read-reg $register]
   }
   update-widget INTMODE $IntModetoW([read-reg DSPIntMode])
   update-widget TRIGCLK $TrigClktoW([read-reg DSPTrigClk])
   clear-busy
   set-footer ""

#    request DA program to get statistics

   EG "register :name '/$RDOUTserver/EXEC.Action' :value '9'"
   EG "register :name '/$RDOUTserver/EXEC.Sig1' :value '0'"

   wait_for_completion "Getting Statistics"

   set state [read-reg DSPState]
   set commands [read-reg DSPCommands]
   set runs [read-reg DSPRuns]

   :modify S1 :label "DSP Program State $state: Data Samples $runs: Commands Processed $commands"

}

proc click-set {} {
    global RDOUTserver

    EG "register :name '/$RDOUTserver/EXEC.Action' :value '7'"
    EG "register :name '/$RDOUTserver/EXEC.Sig1' :value '0'"

    wait_for_completion "Setting Parameters"
}

proc click-intmode {v} {
   global WtoIntMode
   write-reg DSPIntMode $WtoIntMode($v)
}

proc click-trigclk {v} {
   global WtoTrigClk
   write-reg DSPTrigClk $WtoTrigClk($v)
}

proc click arg {
   global RDOUTserver RDOUThdwre
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit]}
   switch $w {
      UPDATE      {click-update}
      SET         {click-set; click-update}
      INTMODE     {click-intmode $v}
      TRIGSRC     {write-reg DSPTrigSrc $v}
      TRIGCLK     {click-trigclk $v}
      TRIGMODE    {write-reg DSPTrigMode $v}
      TRIGCOUNT   {write-reg DSPTrigCount $v}
      SAMPLERATE  {write-reg DSPSampleRate $v}
      TRIGVOLTAGE {write-reg DSPTrigVoltage $v}
   }
}

proc start args {
   global started
   global RDOUTserver RDOUThdwre
   
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
   set RDOUTserver [inform sys inquire RDOUTserver]
   set RDOUThdwre  [inform sys inquire RDOUThdwre]
   
   click-update
}

proc make-frame {} {
  global env

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

   :panel P1 :layout T :padx 10 :pady 10
     :layout V; :row-gap 10

       :choice INTMODE :label "FIFO interrupt trip level" :valuex 28c :w 15 \
          :strings [list "FIFO not empty" "FIFO half full" "FIFO full"]
          
       :choice TRIGSRC :label "Pre/Post trigger source" :valuex 28c :w 15 \
          :strings [list "Use Comparitor" "Use External Gate"]
          
       :choice TRIGCLK :label "Triggering Mode" :valuex 28c :w 15 \
          :strings [list DDS External Crystal]
          
       :choice TRIGMODE :label "Pre/Post Trigger Capture Mode" :valuex 28c :w 15 \
          :strings [list "Edge Negative" "Edge Positive" "Capture Below" "Capture Above" "Continuous"]

   :panel P2 :layout L :padx 10 :pady 10
     :layout V; :row-gap 10
          
       :number TRIGCOUNT :label "Number of Pre Trigger Samples" :valuex 28c :w 8 :min 0 :max 1024
       
       :number SAMPLERATE :label "ADC Sample Rate (MHz)" :valuex 28c :w 8  :min 0 :max 50
       
       :number TRIGVOLTAGE :label "Trigger Threshold (mV)" :valuex 28c :w 8 :min -1000 :max +1000

   :panel P3 :layout L :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10
       
       :message S1 :label "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
       :next-row T

       :button UPDATE
       :button SET
       :button HELP     :command click-help

   :show T
}


