#    most of this task is now taken from the standard application

set VERSION "GRT4P"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME}

send vmecontrol start
send vmecontrol close-frame
after 2000
set RDOUTserverList [inform vmecontrol inquire VMEServerList]

loadsource  sys.common

set S9418_Base 1


if {[llength $RDOUTserverList] != [llength $SpectrumserverList]} then {
    set s [first $SpectrumserverList]
    set SpectrumserverList $s
    for {set i 2} {$i <= [llength $RDOUTserverList]} {incr i} {lappend SpectrumserverList $s$i}
}

set frametitle "MIDAS Data Acquisition Experiment Control (GRT4P)"

set VMEPresent 1
set VMEEnable 1

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""


proc create_s9418_spectra {} {
    global SpectrumServer

   set grt4adcs [expr [inform vmecontrol inquire grt4pcount] * 4]

   set maxmodule [inform vmecontrol inquire s9418count]
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module s9418#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      if {$grt4adcs == 0} {return}   ;# only create spectra for those channels in use 
      incr grt4adcs -1

      set spectrumname [SpectrumName s9418#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum s9418#$i.adc$j"
          clear-footer
          error $m
      } else {
          map_spectrum  "$spectrumname $i [expr $j + 12] 0 0"  
      }
    }
   }
}

proc create_spectra {} {
   global SpectrumServer SpectrumserverList RDOUTserver RDOUTserverList HistMode

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set grt4modules [inform vmecontrol inquire grt4pcount]

   foreach server $SpectrumserverList {
       set SpectrumServer $server

       set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr ($grt4modules * 16) + 32] :name 'Stat' !new)" } m]
       set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr ($grt4modules * 16) + 32] :name 'Rate' !new)" } m]
   }
   map_spectrum "Stat 0 0"

#   for silena 9418

    create_s9418_spectra

#   for the GRP 4 channel module

   set maxmodule [inform vmecontrol inquire grt4pcount]
   set maxadc 4

   for {set i 1} {$i <= $maxmodule} {incr i} {
    set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
    set RDOUTserver $VMEserver
    set k [lsearch $RDOUTserverList $VMEserver]
    set SpectrumServer [lindex $SpectrumserverList $k]
    
    for {set j 0} {$j < $maxadc} {incr j} {

#    histogram used to display the digital trace

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 512 :name 'grt4p#$i.adc$j' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.adc$j"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.adc$j $i $j"
      }

#    histograms for the computed energy 

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'grt4p#$i.E[set j]L' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.E[set j]L"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.E[set j]L $i [expr ($j*2) + 4]"
      }

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'grt4p#$i.E[set j]H' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.E[set j]H"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.E[set j]H $i [expr ($j*2) + 5]"
      }


    }
   }
}

proc define_VME_addresses {} {
    global VMEserver RDOUTserver

    set RDOUTserver $VMEserver

#    load hardware configuration into readout software

   ClearConfig

#    first all the ADCs

   set maxmodule [inform vmecontrol inquire grt4pcount]
   for {set i 1} {$i <= $maxmodule} {incr i} {

      set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
      set z [catch {eg "(register :name '/$VMEserver/grt4p#$i.ChA.DPP_CSR'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register grt4p#$i.ChA.DPP_CSR"
          clear-footer
          error $m
      }
      DefineConfig "4 $i [string range [first $m] 2 end]"
   }

#    the S9418 VME modules

     define_S9418_config

#    then the acquisition control module

     define_sac_config

     PrintConfig
}

proc Custom_PreSetup {args} {define_VME_addresses}

proc redisplay-vme {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]
       set vl [read-registers xvme260#1.PortA]
       if {$vl != ""} then {
           if {[second [first $vl]] == "0x0000"} {:modify VME :value 1} else {:modify VME :value 0}
           return
       }
       set vl [read-registers sac#1.Enable]
       if {$vl != ""} then {
           :modify VME :value [expr [second [first $vl]] & 1]]
           return
       }
       :modify VME :value -1
}

proc Stop_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers xvme260#1.PortA 0

       write-registers sac#1.STR 0
}

proc Go_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers sac#1.MON0 2
       write-registers sac#1.STR 9
       write-registers sac#1.ARM 0

       write-registers xvme260#1.PortA -1
}

proc Reset_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers sac#1.EVCLR 0

       write-registers xvme260#1.PortA 0
}


proc make-test-frame {} {

  global actionlist

  :frame [self frame-name].adv :label "System Configuration, Test and Debug Only" :show-footer T :resize F

    :panel ADVP1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 5

        set i 0
        foreach item $actionlist {
            :button A$i :label "[first $item]" :command "[second $item]" :w 24
            incr i
            if {[expr ($i/4)*4] == $i} {:next-row T}
        }

     :next-row T

    :message M1 :label "program options" :bold T
     :next-row T
      :layout V

       :checkbox SCANTABLE  :label "Enable Scanning Table Controls" :valuex 32c :command do_scantable
       :checkbox USETIME    :label "Use Time Stamp control"         :valuex 32c :command do_option
       :checkbox USECOUNT   :label "Use Event Counter control"      :valuex 32c :command do_option
       :checkbox VERBOSE    :label "Enable Verbose mode"            :valuex 32c :command do_option
       :text     TIMERANGE  :label "Time Stamp compare range"       :valuex 32c :command do_option :w 6

    :next-row T
    :message M2 :label "tape server options" :bold T
      :next-row T
      :layout H

       :col-gap 15
       :text TSSERVER      :label "Tape Server" :w 12 :valuex 12c :command do_tsserver
       :text TSPORT        :label "Port"        :w 8  :command do_tsport
       :hskip 30
       :button TSDEFINE    :label "Setup Connection"  :command do_tsdefine
       :button TSUNDEFINE  :label "Cancel Connection" :command do_tsundefine

    :next-row T
       :button UPDATE :command update-test-frame

  :show T

    update-test-frame
}

set scantable 0
set usetime 0
set usecount 0
set verbose 0
set timerange 100

proc update-test-frame {} {
    global scantable usetime usecount verbose timerange

       :modify TSSERVER    :value [read-register TS_Server]
       :modify TSPORT      :value [read-register TS_Port]

       :modify SCANTABLE  :value $scantable
       :modify USETIME    :value $usetime
       :modify USECOUNT   :value $usecount
       :modify VERBOSE    :value $verbose
       :modify TIMERANGE  :value $timerange
}

proc do_tsserver {w s}  {write-register TS_Server $s}
proc do_tsport   {w v}  {write-register TS_Port $v}
proc do_tsdefine {}     {click-special 3}
proc do_tsundefine {}   {click-special 1}

proc do_scantable {w v} {
    global scantable
 
    set scantable [expr $v*2]
    execute-command 16 "" "$scantable 0 0 0 0 0" 2
}

proc do_option {w v} {
    global usetime usecount verbose timerange

    switch $w {
        VERBOSE   {set verbose $v}
        USETIME   {set usetime $v}
        USECOUNT  {set usecount $v}
        TIMERANGE {set timerange $v}
    }

    execute-command 17 "" "$verbose $usetime $usecount $timerange" 2
}

proc do_clearstats {} {execute-command 7  "Clear Statistics"}

proc enable_scanning {} {execute-command 16 "" "3 1 2 3 4 5" 2}


proc click-histenable {v} {
    global RDOUThdwre RDOUTEnable histEnable
    if {$RDOUTEnable == 0} then {set  histEnable 0} else {set histEnable $v}
    if {$RDOUTEnable != 0} then {write-register $RDOUThdwre.Chistenable [expr $histEnable * 10]}
    redisplay-histenable
}
