# 
#    

#   June 2000   


set debug 0
set started 0

set VERSION "cM62"

set frame SYS
set icontitle  "SysCntl"
set frametitle "MIDAS Data Acquisition System Control (G.R.T)"


set TSEnable 0
set tapestate -1
set tapeassoc ""

set MAX_ADC 6

set histEnable 0
set HistMode 0
set HistChoices [list None All]

set txEnable 0

if {[info exists env(RDOUTServer)]} then {set resource $env(RDOUTServer)} else {set resource cm62}
set RDOUTserver $resource
set RDOUThdwre CM62

if {[info exists env(SASServer)]} then {set resource $env(SASServer)} else {set resource MemSaS}
set SpectrumServer $resource


set RDOUTstate 0
set DelayTicks 6
set MaxTries 20

set DSPProgramFile ""


#    list of menu items and associated actions for the ACTIONS menu button 

set actionlist {                                             \
   {"Enable DA System tracing"          "click-trace 1"}     \
   {"Disable DA System tracing"         "click-trace 0"}     \
   {"transferTxStatus"                  "click-special 1"}   \
   {"transferTxReset"                   "click-special 2"}   \
   {"transferInit"                      "click-special 3"}   \
   {"Enter test mode"                   "click-testmode 0"}  \
   {"Exit test mode"                    "click-testmode 1"}  \
   {"Show EXEC task ID"                 "click-ID"}          \
   {"Show current Error Msg"            "click-ErrMsg"}      \
}


proc starttask {task} {
#    Check if it is necessary to pre-start server interface task

    set started [inform $task inquire started]
    if {$started == 0} then {
       send $task start
       send $task close-frame
    }
}

proc click-tapes {action} {

   starttask tape

#    set tape server state

   send tape click-egstate $action
   redisplay-tapes
}

proc redisplay-tapes {} {
   global tapestate tapeassoc

   set-busy
   starttask tape
  
#    find tape server state

   catch {set tapestate [send tape read-egstate]}
   :modify TAPE :value [expr $tapestate-1]

#    find data stream association

   catch {set tapeassoc [send tape find-assoc-strings 1]}
   :modify STREAM :value "$tapeassoc"

   clear-busy
}
  
proc click-da {v} {
        global debug RDOUTstate TSEnable
        if {$debug} then {puts stderr "setting MIDAS DA to state $v"}
        set RDOUTstate $v
    set z [catch {
        if {$v==0} then {
            set-footer "STOPping MIDAS DA [date]"
                click-rdout $v
                if {$TSEnable != 0} then {click-tapes $v}
            set-footer "MIDAS DA STOPped [date]"
         }
         if {$v==1} then {
            set-footer "GOing MIDAS DA [date]"
                if {$TSEnable != 0} then {click-tapes $v}
                click-rdout $v
            set-footer "MIDAS DA GOing [date]"
         }
     } m ]
}

proc click-setup {} {
    global RDOUTserver
    global histEnable HistMode txEnable

    click-histenable $histEnable
    click-histmode HISTMODE $HistMode
    click-txenable $txEnable

#    stopping DA
    click-da 0

#    initialise readout software
    click-rdout 2

#    load AD40 parameters
    click-rdout 7

#    create and enable histograms

     set-footer "Setting up histogramming"
     set-busy
     unmap_spectra
     delete_spectra
     create_spectra
     map_spectra
     clear-busy
     clear-footer

#    setup readout software
    click-rdout 3

#    update display
    click-redisplay
}

proc map_spectra {} {click-rdout 5}

proc unmap_spectra {} {click-rdout 8}

proc create_spectra {} {
   global SpectrumServer HistMode
   global MAX_ADC

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   for {set i 1} {$i <= $MAX_ADC} {incr i} {

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name 'cM62.adc$i' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum cM62.adc$i"
          clear-footer
          error $m
      }
    }
}

proc delete_spectra {} {
   global SpectrumServer
   global MAX_ADC

   set-footer "Deleting spectra"

   for {set i 1} {$i <= $MAX_ADC} {incr i} {

      set z [catch {eg "(spectrum :path '/$SpectrumServer'  :name 'cM62.adc$i' !delete)" } m]

      if {$z != 0} then {break}
   }

}

proc click-rdout  {v} {
    global debug RDOUTserver RDOUThdwre
    if {$debug} then {insert-debug "click-rdout  state $v"}

    set z [catch {
        if {$v==0} then {
#    Halt
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '1'"
        }
        if {$v==1} then {
#    Go
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '2'"
        }
        if {$v==2} then {
#    Init
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '3'"
        }
        if {$v==3} then {
#    Setup
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '4'"
        }
        if {$v==5} then {
#    Map Spectra
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '5'"
        }
        if {$v==8} then {
#    Unmap Spectra
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '8'"
        }
        if {$v==6} then {
#    Get Param
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '6'"
        }
        if {$v==7} then {
#    Set param
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '7'"
        }
        if {$v==9} then {
#    Get Stats
              EG "register :name '/$RDOUTserver/EXEC.Action' :value '9'"
        }
        EG "register :name '/$RDOUTserver/EXEC.Sig1' :value '0'"
        wait_for_completion ""
    } m ]
    redisplay-rdout
    clear-footer
}

proc redisplay-rdout {} {
   global RDOUTserver  RDOUThdwre

   set state [EG "register :name '/$RDOUTserver/$RDOUThdwre.Sstate' ?value"]
   :modify RDOUT :value $state
}

proc click-tsenable {v} {
   global TSEnable
   set TSEnable $v
   :modify TAPE   :value -1
   :modify STREAM :value ""
   if {$TSEnable != 0} then {redisplay-tapes}
}

proc redisplay-tsenable {} {
   global TSEnable
   :modify TSENABLE :value $TSEnable
}

proc click-txenable  {v} {
    global debug  RDOUTserver RDOUThdwre
    if {$debug} then {puts stderr "click-txenable  state $v"}
    EG "register :name '/$RDOUTserver/$RDOUThdwre.Ctxenable' :value '$v'"
    redisplay-txenable
}

proc redisplay-txenable {} {
   global RDOUTserver RDOUThdwre txEnable
   set txEnable [EG "register :name '/$RDOUTserver/$RDOUThdwre.Ctxenable' ?value"]
   :modify TXENABLE :value $txEnable
}

proc click-histenable {v} {
    global debug  RDOUTserver RDOUThdwre
    if {$debug} then {puts stderr "click-histenable  state $v"}
    EG "register :name '/$RDOUTserver/$RDOUThdwre.Chistenable' :value '$v'"
    redisplay-histenable
}

proc redisplay-histenable {} {
   global RDOUTserver RDOUThdwre histEnable
   set histEnable [EG "register :name '/$RDOUTserver/$RDOUThdwre.Chistenable' ?value"]
   :modify HISTENABLE :value $histEnable
}

proc click-trace {v} {
    global debug  RDOUTserver RDOUThdwre
    if {$debug} then {puts stderr "click-trace  state $v"}
    EG "register :name '/$RDOUTserver/$RDOUThdwre.Ctrace' :value '$v'"
}

proc click-special {a} {
   global RDOUTserver
       EG "register :name '/$RDOUTserver/EXEC.Action' :value '$a'"
       EG "register :name '/$RDOUTserver/EXEC.Sig2' :value '0'"
       wait_for_completion "$a"
}

proc click-testmode {a} {
   global RDOUTserver RDOUThdwre
   global DelayTicks

   if {$a == 0} then {

#    start test mode

       EG "register :name '/$RDOUTserver/$RDOUThdwre.TestMode' :value '1'"
       EG "register :name '/$RDOUTserver/$RDOUThdwre.DelayTicks' :value '$DelayTicks'"
       EG "register :name '/$RDOUTserver/EXEC.Action' :value '5'"
       EG "register :name '/$RDOUTserver/EXEC.Sig2' :value '0'"
   } else {

#    stop test mode

       EG "register :name '/$RDOUTserver/$RDOUThdwre.TestMode' :value '0'"
    }
}

proc click-histmode {w v} {
    global debug  RDOUTserver RDOUThdwre
    global HistMode

    set HistMode $v
    EG "register :name '/$RDOUTserver/$RDOUThdwre.Chistmode' :value '$HistMode'"
}

proc redisplay-histmode {} {
   global RDOUTserver RDOUThdwre HistMode
   set HistMode [EG "register :name '/$RDOUTserver/$RDOUThdwre.Chistmode' ?value"]
   :modify HISTMODE :value $HistMode
}

proc checkinstallation  {} {
   global RDOUTserver VERSION env

   set ID [EG "register :name '/$RDOUTserver/EXEC.ID' ?value"]
   set OS [first $ID]
   set version  [second $ID]
   set release [third $ID]

   if {$version != $VERSION} then {
       set msg " WARNING!  Possible system installation or setup error! \n\
       Require Data Acquisition program $VERSION but discovered \n\
       OS $OS; Program $version; Release $release\n\
       Do you wish to continue?"
       if {[midas-confirmation $msg] == 0} then {return 1}
   }

   set env(RTOS) $OS
   return 0
}

proc click-ID {} {
    global RDOUTserver

    set ID [EG "register :name '/$RDOUTserver/EXEC.ID' ?value"]
    insert-log "Target OS = [first $ID]\nProgram = [second $ID]\nVersion = [third $ID]"

}

proc click-ErrMsg {} {
    global RDOUTserver

    set ErrMsg [EG "register :name '/$RDOUTserver/EXEC.ErrMsg' ?value"]
    insert-log "ErrMsg=$ErrMsg"
}

#   VERSION cM62 only
proc click-dsp_program {v} {
    global RDOUTserver RDOUThdwre DSPProgramFile
    set DSPProgramFile $v
    EG "register :name '/$RDOUTserver/$RDOUThdwre.DSPProgramFile' :value '$DSPProgramFile'"
}

#   VERSION cM62 only
proc redisplay-dsp_program {} {
   global RDOUTserver RDOUThdwre DSPProgramFile
   set DSPProgramFile [EG "register :name '/$RDOUTserver/$RDOUThdwre.DSPProgramFile' ?value"]
   :modify DSP :value $DSPProgramFile
}

proc click-redisplay {} {
  global TSEnable
   set-footer "Updating display"
   catch redisplay-rdout
   catch redisplay-histenable
   catch redisplay-txenable
   catch redisplay-tsenable
   catch redisplay-histmode
   catch redisplay-dsp_program
   if {$TSEnable != 0} then {redisplay-tapes}
   set-footer "Display Updated [date]"
}  
  
proc wait_for_completion {s} {
    global RDOUTserver MaxTries

    set tries 1

    while {$tries < $MaxTries} {

        set-footer "Waiting for action $s to complete ($tries)"

        set z [catch {EG "register :name '/$RDOUTserver/EXEC.Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "Unable to confirm state of action\n$m"
            clear-footer
            return
        }

        if {$m == 0} then {
            clear-footer
            return
        }
        incr tries
        after 500
    }

    midas-warning "Action $s has not completed\nGiving up waiting"
    clear-footer
}

proc click arg {
    global debug actionlist
    if {$debug} then {puts stderr "click with argument \{$arg\}"}
    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set i [lindex $arg 4]

    if {$c==":quit"} then {return [click-quit]}
    switch $w  {
                DA-STOP  {click-da 0}
                DA-GO    {click-da 1}
                DA-SETUP {click-setup}

                RDOUT      {click-rdout $v}
                HISTENABLE {click-histenable $v}
                TXENABLE   {click-txenable $v}
                TSENABLE   {click-tsenable $v}
                TAPE       {click-tapes $v}
                DSP        {click-dsp_program $v}

                CLEARSTATS   {
                  inform stats start
                  inform stats click-clear
                }

                ACTIONS  {
                    set action [lindex $actionlist $i]
                    eval [second $action]
                }

                REDISPLAY {click-redisplay}
                STATISTICS {inform stats start}

                default {puts stdout $arg}
    }
}


proc start args {
   global started
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   if {[checkinstallation] != 0} then {return}

   click-redisplay
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}


proc make-frame {} {
  global env actionlist
  global HistMode HistChoices

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

   :panel P1 :layout L :padx 10 :pady 10
    :layout H; :row-gap 10; :col-gap 10

      :choice RDOUT :label "cM62:" :strings [list Stop Go Init Setup] :valuex 10c :value -1
      :next-row T
      :checkbox HISTENABLE :label "Histogramming Enabled"    :value 0  :valuex 24c
      :next-row T
      :checkbox TXENABLE   :label "Data Transfer Enabled"  :value 0  :valuex 24c
      :next-row T
      :checkbox TSENABLE   :label "Tape Server Enabled"    :value 0  :valuex 24c
      :next-row T
      :choice TAPE :label Tapes: :strings [list Stop Go] :valuex 10c

   :panel P2 :layout L :padx 10 :pady 10
    :layout H; :row-gap 10; :col-gap 10

      :button  DA-STOP  :label  STOP  :w 8 :h 3 :colour green
      :hskip 30
      :button  DA-GO    :label  GO    :w 8 :h 3 :colour red
      :next-row T
      :button DA-SETUP  :label SETUP  :w 26 :h 2 :colour yellow
      :next-row T

      :choice HISTMODE :label Histogram :strings $HistChoices :command click-histmode :value $HistMode
      :next-row T

      :vskip 10
      set menu {}
      foreach item $actionlist {lappend menu [first $item]}
      :menubutton ACTIONS :label "Advanced"  :strings $menu
      :button AD40 :label "AD40 Options" :command "inform ad40 start"

   :panel P3 :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :row-gap 10; :col-gap 10
      :text DSP :label "DSP program" :w 50
      :next-row T

      :message MSG1 :label "Tape System" :bold T
      :next-row T
      :button ENDEXPT :label "End Experiment" :command "do_endrun 1" 
      :button STARTEXPT  :label "Start Experiment"  :command "inform tape start"
      :button NEXTRUN :label "Next Run" :command "do_newrun 1"
      :button TAPECONTROL :label "Tape Control" :command "inform tape start"
      :next-row T
      :message MSG2 :label "Current Data Stream Association:" :bold T
   :row-gap 5; :next-row T
      :text STREAM :label "Stream 1:" :valuex 15c :w 42 :deaf T
   :row-gap 10; :next-row T
      :button REDISPLAY    :label Redisplay
      :button STATISTICS   :label Statistics  :command "inform stats start"
      :button SB      :label "Spectrum Browser" :command "inform sb start"
      :button SP      :label "Spectrum Viewer"  :command "inform sd start"

   :show T

}

loadsource endrun+newrun
